unit miscProc;
interface

procedure ddCD(cmdString : string);
procedure ddCopy(cmdString : string);
procedure ddDel(cmdString : string);
procedure ddDeltree(cmdString : string);
procedure ddDir(cmdString : string);
procedure ddExec(cmdString : string; dispResult : boolean);
procedure ddHelp(cmdString : string);
procedure ddMove(cmdString : string);
procedure ddMd(cmdString : string);
procedure ddRen(cmdString : string);
procedure ddRecv;
procedure ddSend(cmdString : string);
procedure ddType(cmdString : string);
procedure ddVersion;
procedure drawPrompt;
procedure processCmd(s : string);
procedure trim(var s : string);
procedure updateCmdBuf(cmd : string);

implementation

uses dos, crt, xdoor, xmisc, xfileio, miscfunc;

procedure clrDir(path : pathStr);
  var dirInfo : searchRec;
      f1 : file;
      path2 : pathStr;
      s : string;
  begin
    findFirst(path + '\*.*', AnyFile, dirInfo);
    while (dosError = 0) do begin
      if (dirInfo.name[1] <> '.') and (dirInfo.attr <> VolumeID) then
        if ((dirInfo.attr and Directory) = Directory) then begin
          path2 := path + '\' + dirInfo.name;
          clrDir(path2);
      end
      else if ((dirInfo.attr and VolumeID) <> VolumeID) then begin
        assign(f1, path + '\' + dirInfo.name);
        erase(f1);
      end;
      findNext(dirInfo);
    end;
    {$I-} rmDir(path); {$I+}
    if (IOResult <> 0) then coutln('|07Directory ' + path + ' not found.');
  end;

procedure copyFile(origFile, newFile : string);
  var f1, f2 : file;
      numRead, numWritten, attr : word;
      buf : array[1..2048] of char;
  begin
    assign(f1, origFile);
    reset(f1, 1);
    assign(f2, newFile);
    rewrite(f2, 1);
    repeat
      blockRead(f1, buf, sizeOf(buf), numRead);
      blockWrite(f2, buf, numRead, numWritten);
    until (numRead = 0) or (numWritten <> numRead);
    close(f1);
    close(f2);
  end;

procedure shell(cmdString : string);
  begin
    swapvectors;
    exec(getenv('COMSPEC'), '/c ' + cmdString);
    swapvectors;
  end;

procedure ddCD(cmdString : string);
  begin
    delete(cmdString, 1, 2);
    trim(cmdString);
    if (cmdString = '') then begin
      ddHelp('help cd');
      exit
    end;
    {$I-} chdir(cmdString); {$I+}
    if (IOResult <> 0) then coutln('|07Path not found.');
  end;

procedure ddCopy(cmdString : string);
  var origFile, newFile : string[255];
      key : char;
      f1 : file;
      attr : word;
  begin
    delete(cmdString, 1, 4);
    trim(cmdString);
    if (numToken(cmdString, ' ') <> 2) then begin
      ddHelp('help copy');
      exit;
    end;
    origFile := getToken(cmdString, ' ', 1);
    newFile := getToken(cmdSTring, ' ', 2);
    if (not fileExists(origFile)) then begin
      nl;
      coutln('|07' + origFile + ' does not exist.');
      exit;
    end;
    if (fileExists(newFile)) then begin
      nl;
      coutln('|07WARNING: ' + newFile + ' already exists.');
      cout('Overwrite? ');
      key := inKey;
      if (upcase(key) = 'N') then begin
        nl;
        exit;
      end
      else nl;
    end;
    assign(f1, newFile);
    getFAttr(f1, attr);
    if (attr = directory) then newFile := newFile + '\' + origFile;
    copyFile(origFile, newFile);
    coutln('|07Copied ' + origFile + ' to ' + newFile);
  end;

procedure ddDel(cmdString : string);
  var f1 : file;
      dirInfo : searchRec;
      attr : word;
  begin
    delete(cmdString, 1, 3);
    trim(cmdString);
    if (numToken(cmdString, ' ') <> 1) then begin
      ddHelp('help del');
      exit;
    end;
    if (pos('*', cmdString) = 0) then begin
      if (isDir(cmdString)) then begin
        nl;
        coutln('|07' + cmdString + ' is a directory. Use |15DELTREE|07.');
        exit;
      end;
      assign(f1, cmdString);
      {$I-} erase(f1); {$I+}
      nl;
      if (IOResult <> 0) then coutln('|07Could not delete ' + cmdString)
      else coutln('|07Deleted file ' + cmdString);
    end
    else begin
      if (pos('.', cmdString) = 0) then cmdString := cmdString + '.*';
      if (cmdString = '.*') then cmdString := '*.*';
      findFirst(cmdString, Anyfile, dirInfo);
      while (dosError = 0) do begin
        if (dirInfo.attr = Archive) then begin
          assign(f1, dirInfo.name);
          erase(f1);
        end;
        findNext(dirInfo);
      end;
    end;
  end;

procedure ddDeltree(cmdString : string);
  begin
    delete(cmdString, 1, 7);
    trim(cmdString);
    if (numToken(cmdString, ' ') <> 1) then begin
      ddHelp('help deltree');
      exit;
    end;
    if (isDir(cmdString)) then clrDir(cmdString)
    else begin
      nl;
      coutln(cmdString + ' is not a directory.');
    end;
  end;

procedure ddDir(cmdString : string);
  var dirInfo : searchRec;
      pause, altOutput : boolean;
      origPath, curPath : string[255];
      curDrive, key : char;
      b, dirNum, fileNum, scrLine : byte;
      totalFsize : longint;
      dt : dateTime;
      fileList, dirList : array[1..255] of searchRec;
  begin
    pause := false;
    altOutput := false;
    dirNum := 0;
    fileNum := 0;
    totalFsize := 0;
    delete(cmdString, 1, 3);
    getDir(0, origPath);
    if (pos('/P', upString(cmdString)) <> 0) then begin
      pause := true;
      delete(cmdString, pos('/P', upString(cmdString)), 2);
    end;
    if (pos('/W', upString(cmdString)) <> 0) then begin
      altOutput := true;
      delete(cmdString, pos('/W', upString(cmdString)), 2);
    end;
    trim(cmdString);
    if (isDir(cmdString)) then begin
      chDir(cmdString);
      cmdString := '*.*';
    end;
    getDir(0, curPath);
    curDrive := curPath[1];
    if (cmdString = '') then cmdString := '*.*'
    else if (cmdString[1] = '.') then cmdString := '*' + cmdString
    else if (pos('.', cmdString) = 0) then cmdString := cmdString + '.*';
    findFirst(cmdString, anyfile, dirInfo);
    while (dosError = 0) do begin
      if (dirInfo.attr = directory) then begin
        inc(dirNum);
        dirList[dirNum] := dirInfo;
      end
      else if (dirInfo.attr = archive) then begin
        inc(fileNum);
        fileList[fileNum] := dirInfo;
        totalFsize := totalFsize + dirInfo.size;
      end;
      findNext(dirInfo);
    end;
    if (getVol(curDrive) <> '') then
      coutln('Volume in drive ' + curDrive + ' is ' + getVol(curDrive))
    else coutln('Volume in drive ' + curDrive + ' has no label');
    coutln('Directory of ' + curPath);
    nl;
    if (not altOutput) then begin
      scrLine := 3;
      for b := 1 to dirNum do begin
        cout(padb(dirList[b].name, 13) + padf('<DIR>', 14));
        setPosX(30);
        unpackTime(dirList[b].time, dt);
        cout(leadZero(dt.month) + '-' + leadZero(dt.day) + '-' + itos(dt.year));
        posRight(1);
        coutln(doTime(dt.hour, dt.min, dt.sec));
        inc(scrLine);
        if (scrLine > 21) and (pause) then begin
          cout('Press any key to continue...');
          key := inkey;
          nl;
          scrLine := 0;
        end;
      end;
      for b := 1 to fileNum do begin
        cout(padb(fileList[b].name, 13) + padf(itosc(fileList[b].size), 14));
        setPosX(30);
        unpackTime(fileList[b].time, dt);
        cout(leadZero(dt.month) + '-' + leadZero(dt.day) + '-' + itos(dt.year));
        posRight(1);
        coutln(doTime(dt.hour, dt.min, dt.sec));
        inc(scrLine);
        if (scrLine > 21) and (pause) then begin
          cout('Press any key to continue...');
          key := inkey;
          nl;
          scrLine := 0;
        end;
      end;
    end
    else begin
      scrLine := 3;
      for b := 1 to dirNum do begin
        cout(padb('[' + dirList[b].name + ']', 16));
        if ((b mod 5) = 0) then inc(scrLine);
        if (scrLine > 21) and (pause) then begin
          nl;
          cout('Press any key to continue...');
          key := inkey;
          nl;
          scrLine := 0;
        end;
      end;
      for b := 1 to fileNum do begin
        cout(padb(fileList[b].name, 16));
        if ((b mod 5) = 0) then inc(scrLine);
        if (scrLine > 21) and (pause) then begin
          nl;
          cout('Press any key to continue...');
          key := inkey;
          nl;
          scrLine := 0;
        end;
      end;
      nl;
    end;
    setPosX(7);
    cout(padf(itosc(fileNum) + ' file(s)', 11));
    setPosX(21);
    coutln(padf(itosc(totalFsize), 13) + ' bytes');
    setPosX(7);
    coutln(padf(itosc(dirNum) + ' dir(s)', 10));
    nl;
  end;


{procedure ddDir(cmdString : string; pause : boolean);
  var dirInfo : searchRec;
      curPath : string;
      curDrive : char;
      dirNum, fileNum : byte;
      totalFsize : longint;

  begin
    getDir(0, curPath);
    curDrive := curPath[1];
    dirNum := 0;
    fileNum := 0;
    totalFsize := 0;
    coutln('|07');
    if (getVol(curDrive) <> '') then
      coutln('|07Volume in drive ' + curDrive + ' is ' + getVol(curDrive))
    else
      coutln('|07Volume in drive ' + curDrive + ' has no label');
    coutln('|07Directory of ' + curPath);
    coutln('|07');
    delete(cmdString, 1, 3);
    trim(cmdString);
    if (cmdString = '') then cmdString := '*.*'
    else if (pos('.', cmdString) = 0) then cmdString := cmdString + '.*';
    findFirst(cmdString, Anyfile, dirInfo);
    while dosError = 0 do begin
      if (dirInfo.attr = Directory) then begin
        cout(padb(dirInfo.Name, 13));
        coutln(padf('<DIR>', 11));
        inc(dirNum);
      end;
      findNext(dirInfo);
    end;
    findFirst(cmdString, anyfile, dirInfo);
    while dosError = 0 do begin
      if (dirInfo.attr = Archive) then begin
        if (pos('.', dirInfo.name) = 0) then
          cout(padb(dirInfo.Name, 13))
        else begin
          cout(padb(getToken(dirInfo.name, '.', 1), 8));
          cout(' ' + getToken(dirInfo.name, '.', 2));
        end;
        coutln(padf(itosc(dirInfo.size), 14));
        totalFsize := totalFsize + dirInfo.size;
        inc(fileNum);
      end;
      findNext(dirInfo);
    end;
    cout('       ' + padf(itosc(fileNum) + ' file(s)', 11));
    setPosX(21);
    coutln(padf(itosc(totalFsize), 13) + ' bytes');
    coutln('       ' + padf(itos(dirNum) + ' dir(s)', 10));
    nl;
  end;
}

procedure ddExec(cmdString : string; dispResult : boolean);
  var f1 : file;
  begin
    delete(cmdString, 1, 1);
    trim(cmdString);
    if (numToken(cmdString, ' ') < 1) then begin
      if (dispResult) then ddHelp('help !')
      else ddHelp('help &');
      exit;
    end;
    if (dispResult) then cmdString := cmdString + ' > dd9378.tmp';
    swapvectors;
    exec(getenv('COMSPEC'), '/c ' + cmdString);
    swapvectors;
    nl;
    case dosError of
      0 : coutln('Operation completed successfully.');
      2 : coutln('File not found.');
      3 : coutln('Path not found.');
      5 : coutln('Access denied.');
      6 : coutln('Invalid handle.');
      8 : coutln('Not enough memory.');
     10 : coutln('Invalid environment.');
     11 : coutln('Invalid format.');
     18 : coutln('No more files.');
    end;
    if (dispResult) then begin
      showfile('dd9378.tmp', true);
      assign(f1, 'dd9378.tmp');
      erase(f1);
    end;
  end;

procedure ddHelp(cmdString : string);
  begin
    if (numToken(cmdString, ' ') = 1) then begin
      nl;
      coutln('|07DosDoor v3.0 - A Demonic Production');
      coutln('|07Use |15Help <command> |07for help on a specific command.');
      coutln('|07     ' + padb('!',10) + padb('&',10) + padb('CD',10) + padb('CLS',10) + 'DEL');
      coutln('|07     ' + padb('DELTREE',10) + padb('DIR',10) + padb('COPY',10) + padb('MOVE',10) + 'REN');
      coutln('|07     ' + padb('RECV',10) + padb('SEND',10) + padb('MD',10) + padb('TYPE',10) + 'EXIT');
    end
    else if (numToken(cmdString, ' ') = 2) then begin
      delete(cmdString, 1, 4);
      trim(cmdString);
      nl;
      coutln('|07Usage:');
      if (upString(cmdString) = '!') then begin
          coutln('|15!|07<command>');
          coutln('|07This will redirect the output of <command> to a temporary file and then display it.');
        end
      else if (upString(cmdString) = '&') then begin
          coutln('|15&|07<command>');
          coutln('|07This will execute <command>');
        end
      else if (upString(cmdString) = 'CD') then begin
          coutln('|15CD |07<directory>');
          coutln('|07Changes current directory to <directory>');
        end
      else if (upString(cmdString) = 'CLS') then begin
          coutln('|15CLS');
          coutln('|07This will clear the screen.');
        end
      else if (upString(cmdString) = 'COPY') then begin
          coutln('|15COPY |07<file1> <file2>');
          coutln('|07Copies <file1> to <file2>');
        end
      else if (upString(cmdString) = 'DEL') then begin
          coutln('|15DEL |07<files>');
          coutln('|07Deletes <files>. Wildcards may be used.');
        end
      else if (upString(cmdString) = 'DELTREE') then begin
          coutln('|15DELTREE |07<directory>');
          coutln('|07Deletes <directory> recursively.');
        end
      else if (upString(cmdString) = 'DIR') then begin
          coutln('|15DIR |07[file]');
          coutln('|07Lists [file]. Wildcards may be used. If [file] is not given, *.* will be used.');
        end
      else if (upString(cmdString) = 'MOVE') then begin
          coutln('|15MOVE |07<file1> <file2>');
          coutln('|07Moves <file1> to <file2>');
        end
      else if (upString(cmdString) = 'REN') then begin
          coutln('|15REN |07<file1> <file2>');
          coutln('|07Renames <file1> to <file2>');
        end
      else if (upString(cmdString) = 'RECV') then begin
          coutln('|15RECV');
          coutln('|07Starts CEXYZ in z-modem receive mode.');
        end
      else if (upString(cmdString) = 'SEND') then begin
          coutln('|15SEND |07<file1> <file2> ...');
          coutln('|07Starts CEXYZ in z-modem send mode.');
        end
      else if (upString(cmdString) = 'MD') then begin
          coutln('|15MD |07<directory>');
          coutln('|07Creates <directory>');
        end
      else if (upString(cmdString) = 'TYPE') then begin
          coutln('|15TYPE |07<file>');
          coutln('|07Displays <file>');
        end
      else if (upString(cmdString) = 'EXIT') then begin
          coutln('|15EXIT');
          coutln('|07Exits DosDoor.');
        end;
    end;
  end;

procedure ddMove(cmdString : string);
  var f1 : file;
      oldN, newN : string[13];
  begin
    delete(cmdString, 1, 4);
    trim(cmdString);
    if (numToken(cmdString, ' ') <> 2) then begin
      ddHelp('help move');
      exit;
    end;
    oldN := getToken(cmdString, ' ', 1);
    newN := getToken(cmdString, ' ', 2);
    if (isDir(newN)) then newN := newN + '\' + oldN;
    assign(f1, oldN);
    {$I-} rename(f1, newN); {$I+}
    nl;
    if (IOResult <> 0) then coutln('|07Could not move ' + oldN)
    else coutln('|07' + oldN + ' => ' + newN + ' [ok]');
  end;

procedure ddMd(cmdString : string);
  begin
    delete(cmdString, 1, 2);
    trim(cmdString);
    if (numToken(cmdString, ' ') <> 1) then begin
      ddHelp('help md');
      exit;
    end;
    {$I-} mkdir(cmdString); {$I+}
    if (IOResult <> 0) then begin
      nl;
      coutln('|07Could not create directory ' + cmdString);
    end;
  end;

procedure ddRen(cmdString : string);
  begin
    delete(cmdString, 1, 3);
    trim(cmdString);
    if (numToken(cmdString, ' ') <> 1) then begin
      ddHelp('help ren');
      exit;
    end;
    ddMove('MOVE ' + cmdString);
  end;

procedure ddRecv;
  var s : string;
  begin
    if (fileExists(ddCEXYZ) = FALSE) then begin
      nl;
      cout('|07Could not find ' + ddCEXYZ + '! Please set the correct');
      coutln(' path in DOSDOOR.CFG');
    end
    else begin
      s := ddCEXYZ + ' /T7 /P' + itos(port) + ' /L' + itos(lockedBaud) + ' /B' + itos(userBaud) + ' /Rz';
      shell(s);
    end;
  end;

procedure ddSend(cmdString : string);
  var s : string;
  begin
    if (fileExists(ddCEXYZ) = FALSE) then begin
      nl;
      cout('|07Could not find ' + ddCEXYZ + '! Please set the correct');
      coutln(' path in DOSDOOR.CFG');
    end
    else begin
      delete(cmdString, 1, 4);
      trim(cmdString);
      if (numToken(cmdString, ' ') < 1) then begin
        ddHelp('help send');
        exit;
      end;
      s := ddCEXYZ + ' /T7 /P' + itos(port) + ' /L' + itos(lockedBaud) + ' /B' + itos(userBaud) + ' /Sz ' + cmdString;
      shell(s);
    end;
  end;

procedure ddType(cmdString : string);
  var pause : boolean;
  begin
    pause := false;
    delete(cmdString, 1, 4);
    trim(cmdString);
    if (pos('/p', cmdString) <> 0) then begin
      pause := true;
      delete(cmdString, pos('/p', cmdString), 2);
      trim(cmdString);
    end;
    if (numToken(cmdString, ' ') <> 1) then begin
      ddHelp('help type');
      exit;
    end;
    if (isDir(cmdString)) then begin
      nl;
      coutln(cmdString + ' is a directory!');
    end
    else showFile(cmdString, pause);
  end;

procedure ddVersion;
  begin
    nl;
    coutln('|07DosDoor v3.0');
    coutln('|07   Written by Stoned Milita of Demonic Productions');
    nl;
  end;

procedure drawPrompt;
  var s : string[255];
  begin
    getDir(0, s);
    cOut('|07' + s + '>');
  end;

procedure processCmd(s : string);
  begin
    trim(s);
    if (upString(leftStr(s, 2)) = 'CD') then ddCD(s)
    else if (upstring(leftStr(s, 1)) = '!') then ddExec(s, true)
    else if (upstring(leftStr(s, 1)) = '&') then ddExec(s, false)
    else if (upstring(leftStr(s, 3)) = 'CLS') then cClrScr
    else if (upString(leftStr(s, 4)) = 'COPY') then ddCopy(s)
    else if (upstring(leftStr(s, 7)) = 'DELTREE') then ddDeltree(s)
    else if (upstring(leftStr(s, 3)) = 'DEL') then ddDel(s)
    else if (upstring(leftStr(s, 3)) = 'DIR') then ddDir(s)
    else if (upstring(leftStr(s, 4)) = 'HELP') then ddHelp(s)
    else if (upstring(leftStr(s, 4)) = 'MOVE') then ddMove(s)
    else if (upstring(leftStr(s, 2)) = 'MD') then ddMd(s)
    else if (upstring(leftStr(s, 3)) = 'REN') then ddRen(s)
    else if (upstring(leftStr(s, 4)) = 'RECV') then ddRecv
    else if (upstring(leftStr(s, 4)) = 'SEND') then ddSend(s)
    else if (upstring(leftStr(s, 4)) = 'TYPE') then ddType(s)
    else if (upstring(leftStr(s, 3)) = 'VER') then ddVersion
    else if (upString(s) = 'EXIT') then ddDone := true
    else if (s = '') then cout('')
    else coutln('|07Bad command or filename');
    updateCmdBuf(s);
  end;

procedure trim(var s : string);
  var done : boolean;
  begin
    done := false;
    repeat
      if (s[1] = ' ') then delete(s, 1, 1)
      else done := true;
    until done;
    done := false;
    repeat
      if (s[length(s)] = ' ') then delete(s, length(s), 1)
      else done := true;
    until done;
  end;

procedure updateCmdBuf(cmd : string);
  var b : byte;
  begin
    for b := 2 to 15 do ddCmdBuf[b-1] := ddcmdBuf[b];
    ddCmdBuf[15] := cmd;
  end;

end.